@@ -194,6 +194,7 @@ typedef struct {
 
 	/* Accumulator for storing arrays after the expression */
 	GSList		*arrays;
+	GnmExprTop const *array_texpr;
 } PolishData;
 
 static void write_node (PolishData *pd, GnmExpr const *expr,
@@ -231,7 +232,7 @@ xl_map_char_to_type (char t)
 		return XL_ARRAY;
 	if (t == 'v')
 		return XL_ROOT;
-	g_warning ("unknown op class '%c' assuming val", t);
+	g_warning ("unknown op class '%c' assuming val", t ? t : '-');
 	return XL_VAL;
 }
 
@@ -553,7 +554,9 @@ write_funcall (PolishData *pd, GnmExpr const *expr,
 		min_args = ef->efunc->min_args;
 		max_args = ef->efunc->max_args;
 		func_idx = ef->efunc->idx;
-		func_type = xl_map_char_to_type (ef->efunc->type);
+		func_type = ef->efunc->type
+			? xl_map_char_to_type (ef->efunc->type)
+			: XL_VAL; // Assumption
 		arg_types = ef->efunc->known_args;
 	} else {
 		min_args = max_args = expr->func.argc;
@@ -861,18 +864,16 @@ write_node (PolishData *pd, GnmExpr const *expr, int paren_level,
 		guint8 data[5];
 		int x, y, ptg;
 
-		if (GNM_EXPR_OP_ARRAY_ELEM == op) {
-			x = expr->array_elem.x;
-			y = expr->array_elem.y;
-		} else
+		if (!gnm_expr_top_is_array_elem (pd->array_texpr, &x, &y))
 			x = y = 0;
 
 		ptg = FORMULA_PTG_EXPR;
 		if (pd->sheet != NULL) {
-			GnmExprArrayCorner const *corner = gnm_cell_is_array_corner (
-				sheet_cell_get (pd->sheet, pd->col - x, pd->row - y));
-			if (NULL != corner &&
-			    gnm_expr_is_data_table (corner->expr, NULL, NULL))
+			GnmCell const *ccell =
+				sheet_cell_get (pd->sheet, pd->col - x, pd->row - y);
+			if (ccell &&
+			    gnm_expr_top_is_array_corner (ccell->base.texpr) &&
+			    gnm_expr_is_data_table (gnm_expr_top_get_array_expr (ccell->base.texpr), NULL, NULL))
 				ptg = FORMULA_PTG_TBL;
 		}
 
@@ -978,7 +979,7 @@ write_arrays (PolishData *pd)
 
 guint32
 excel_write_array_formula (ExcelWriteState *ewb,
-			   GnmExprArrayCorner const *array,
+			   GnmExprTop const *texpr,
 			   Sheet *sheet, int fn_col, int fn_row)
 {
 	PolishData pd;
@@ -986,19 +987,20 @@ excel_write_array_formula (ExcelWriteState *ewb,
 	guint32 len;
 
 	g_return_val_if_fail (ewb, 0);
-	g_return_val_if_fail (array, 0);
+	g_return_val_if_fail (texpr, 0);
 
 	pd.col     = fn_col;
 	pd.row     = fn_row;
 	pd.sheet   = sheet;
 	pd.ewb     = ewb;
 	pd.arrays  = NULL;
+	pd.array_texpr = texpr;
 	pd.context = CTXT_ARRAY;
 	pd.use_name_variant = FALSE;
 	pd.allow_sheetless_ref = TRUE;
 
 	start = ewb->bp->curpos;
-	write_node (&pd, array->expr, 0, XL_ROOT);
+	write_node (&pd, gnm_expr_top_get_array_expr (texpr), 0, XL_ROOT);
 	len = ewb->bp->curpos - start;
 
 	write_arrays (&pd);
@@ -1030,6 +1032,12 @@ excel_write_formula (ExcelWriteState *ewb, GnmExprTop const *texpr,
 	pd.arrays  = NULL;
 	pd.allow_sheetless_ref = TRUE;
 
+	if (gnm_expr_top_is_array_corner (texpr) ||
+	    gnm_expr_top_is_array_elem (texpr, NULL, NULL))
+		pd.array_texpr = texpr;
+	else
+		pd.array_texpr = NULL;
+
 	switch (context) {
 	case EXCEL_CALLED_FROM_CELL:
 		pd.context = CTXT_CELL;
