@@ -26,8 +26,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <gnumeric-config.h>
 #include <gnumeric.h>
@@ -41,7 +40,6 @@
 #include <workbook.h>
 #include <sheet.h>
 #include <cell.h>
-#include <expr-impl.h>
 #include <string.h>
 
 #ifdef WIN32
@@ -148,7 +146,7 @@ static const char *
 gnm_value_type_name (const GnmValue*g)
 {
 	if (NULL != g) {
-		switch (g->type) {
+		switch (g->v_any.type) {
 		CASE(VALUE_EMPTY);
 		CASE(VALUE_BOOLEAN);
 		CASE(VALUE_FLOAT);
@@ -277,7 +275,7 @@ copy_construct_xloper_from_gnm_value (XLOPER*out, const GnmValue*in,
 	out->val.num = 0;
 
 	if (NULL != in) {
-		switch (in->type) {
+		switch (in->v_any.type) {
 		case VALUE_EMPTY:
 			out->xltype = xltypeNil;
 			break;
@@ -347,7 +345,7 @@ copy_construct_xloper_from_gnm_value (XLOPER*out, const GnmValue*in,
 			break;
 		}
 		default:;
-			g_warning (_("Unsupported GnmValue type (%d)"),in->type);
+			g_warning (_("Unsupported GnmValue type (%d)"),in->v_any.type);
 		}
 	}
 }
@@ -491,17 +489,17 @@ genericXLLFunction (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 	GnmValue *g = NULL;
 	guint i,m;
 	const XLLFunctionInfo*info = NULL;
+	GnmFunc const *gfunc = gnm_eval_info_get_func (ei);
+
 	g_assert (NULL != xll_function_info_map);
-	info=g_tree_lookup (xll_function_info_map,ei->func_call->func->name);
+	info=g_tree_lookup (xll_function_info_map, gfunc->name);
 	g_assert (NULL != info);
-	m=ei->func_call->argc;
-	if ( m > MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS )
-		m = MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS;
+	m = gnm_eval_info_get_arg_count (ei);
+	m = MAX (m, MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS);
 	for (i = 0; i < m; ++i)
 		copy_construct_xloper_from_gnm_value (x+i,argv[i],ei);
 	m = info->number_of_arguments;
-	if ( m > MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS )
-		m = MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS;
+	m = MAX (m, MAXIMUM_NUMBER_OF_EXCEL_FUNCTION_ARGUMENTS);
 	for (; i < m; ++i)
 		x[i].xltype=xltypeMissing;
 	func = (XLLFunctionWithVarArgs)info->xll_function;
