@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <gnumeric-config.h>
@@ -30,6 +29,7 @@
 #include <sheet.h>
 #include <workbook.h>
 #include <value.h>
+#include <criteria.h>
 #include <collect.h>
 #include <rangefunc.h>
 #include <gnm-i18n.h>
