@@ -448,7 +448,7 @@ gnumeric_time (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 	time = (hours * 3600 + minutes * 60 + seconds) / DAY_SECONDS;
 	time -= gnm_fake_floor (time);
 
-	return make_date (value_new_float (time));
+	return value_new_float (time);
 }
 
 /***************************************************************************/
@@ -647,7 +647,7 @@ static GnmFuncHelp const help_weekday[] = {
         { GNM_FUNC_HELP_NAME, F_("WEEKDAY:day-of-week") },
         { GNM_FUNC_HELP_ARG, F_("date:date serial value")},
         { GNM_FUNC_HELP_ARG, F_("method:numbering system, defaults to 1")},
-	{ GNM_FUNC_HELP_DESCRIPTION, F_("The WEEKDAY function returns the day-of-week of @{date}.  The value of @{method} determines how days are numbered; it defaults to 1. ") },
+	{ GNM_FUNC_HELP_DESCRIPTION, F_("The WEEKDAY function returns the day-of-week of @{date}.  The value of @{method} determines how days are numbered; it defaults to 1.") },
 	{ GNM_FUNC_HELP_NOTE, F_("If @{method} is 1, then Sunday is 1, Monday is 2, etc.") },
 	{ GNM_FUNC_HELP_NOTE, F_("If @{method} is 2, then Monday is 1, Tuesday is 2, etc.") },
 	{ GNM_FUNC_HELP_NOTE, F_("If @{method} is 3, then Monday is 0, Tuesday is 1, etc.") },
