@@ -20,8 +20,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  * Many thanks to Harlan Grove for his excellent characterization and writeup
  * of the multitude of different potential arguments across the various
@@ -43,7 +42,6 @@
 #include <style-font.h>
 #include <value.h>
 #include <expr.h>
-#include <expr-impl.h>
 #include <workbook.h>
 #include <sheet-style.h>
 #include <number-match.h>
@@ -1379,8 +1377,8 @@ gnumeric_info (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 		 * window, based on the current scrolling position.
 		 */
 		return value_new_error (ei->pos, _("Unimplemented"));
-#ifdef HAVE_UNAME
 	} else if (!g_ascii_strcasecmp (info_type, "osversion")) {
+#ifdef HAVE_UNAME
 		/* Current operating system version, as text.  */
 		struct utsname unamedata;
 
@@ -1393,11 +1391,11 @@ gnumeric_info (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 						     unamedata.release);
 			return value_new_string_nocopy (tmp);
 		}
+#elif defined(G_OS_WIN32)
+		/* fake XP */
+		return value_new_string ("Windows (32-bit) NT 5.01");
 #else
-#ifdef G_OS_WIN32
-	} else if (!g_ascii_strcasecmp (info_type, "osversion")) {
-		return value_new_string ("Windows (32-bit) NT 5.01");	/* fake XP */
-#endif
+		// Nothing -- go to catch-all
 #endif
 	} else if (!g_ascii_strcasecmp (info_type, "recalc")) {
 		/* Current recalculation mode; returns "Automatic" or "Manual".  */
@@ -1407,8 +1405,8 @@ gnumeric_info (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 	} else if (!g_ascii_strcasecmp (info_type, "release")) {
 		/* Version of Gnumeric (Well, Microsoft Excel), as text.  */
 		return value_new_string (GNM_VERSION_FULL);
-#ifdef HAVE_UNAME
 	} else if (!g_ascii_strcasecmp (info_type, "system")) {
+#ifdef HAVE_UNAME
 		/* Name of the operating environment.  */
 		struct utsname unamedata;
 
@@ -1416,11 +1414,10 @@ gnumeric_info (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 			return value_new_error (ei->pos, _("Unknown system"));
 		else
 			return value_new_string (unamedata.sysname);
-#else
-#ifdef G_OS_WIN32
-	} else if (!g_ascii_strcasecmp (info_type, "system")) {
+#elif defined(G_OS_WIN32)
 		return value_new_string ("pcdos");	/* seems constant */
-#endif
+#else
+		// Nothing -- go to catch-all
 #endif
 	} else if (!g_ascii_strcasecmp (info_type, "totmem")) {
 		/* Total memory available, including memory already in use, in
@@ -1688,11 +1685,20 @@ static GnmFuncHelp const help_isref[] = {
 static GnmValue *
 gnumeric_isref (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 {
+	GnmValue *v;
+	gboolean res;
+
 	if (argc != 1)
 		return value_new_error (ei->pos,
 					_("Invalid number of arguments"));
 
-	return value_new_bool (GNM_EXPR_GET_OPER (argv[0]) == GNM_EXPR_OP_CELLREF);
+	v = gnm_expr_eval (argv[0], ei->pos,
+			   GNM_EXPR_EVAL_PERMIT_NON_SCALAR |
+			   GNM_EXPR_EVAL_WANT_REF);
+	res = VALUE_IS_CELLRANGE (v);
+	value_release (v);
+
+	return value_new_bool (res);
 }
 
 /***************************************************************************/
@@ -1791,7 +1797,7 @@ gnumeric_type (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 static GnmFuncHelp const help_getenv[] = {
         { GNM_FUNC_HELP_NAME, F_("GETENV:the value of execution environment variable @{name}")},
         { GNM_FUNC_HELP_ARG, F_("name:the name of the environment variable")},
-	{ GNM_FUNC_HELP_NOTE, F_("If a variable called @{name} does not exist, #N/A! will be returned.") },
+	{ GNM_FUNC_HELP_NOTE, F_("If a variable called @{name} does not exist, #N/A will be returned.") },
 	{ GNM_FUNC_HELP_NOTE, F_("Variable names are case sensitive.") },
 	{ GNM_FUNC_HELP_EXAMPLES, "=GETENV(\"HOME\")" },
         { GNM_FUNC_HELP_END}
@@ -1801,9 +1807,9 @@ static GnmValue *
 gnumeric_getenv (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 {
 	char const *var = value_peek_string (argv[0]);
-	char const *val = getenv (var);
+	char const *val = g_getenv (var);
 
-	if (val)
+	if (val && g_utf8_validate (val, -1, NULL))
 		return value_new_string (val);
 	else
 		return value_new_error_NA (ei->pos);
@@ -1830,7 +1836,7 @@ gnumeric_get_link (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 		GnmCellRef const * a = &v->v_range.cell.a;
 		GnmCellRef const * b = &v->v_range.cell.b;
 		Sheet *sheet;
-		GnmHLink *link;
+		GnmHLink *lnk;
 		GnmCellPos pos;
 
 		if (a->col != b->col || a->row != b->row || a->sheet !=b->sheet)
@@ -1838,10 +1844,10 @@ gnumeric_get_link (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 
 		sheet = (a->sheet == NULL) ? ei->pos->sheet : a->sheet;
 		gnm_cellpos_init_cellref (&pos, a, &(ei->pos->eval), sheet);
-		link = gnm_sheet_hlink_find (sheet, &pos);
+		lnk = gnm_sheet_hlink_find (sheet, &pos);
 
-		if (link)
-			return value_new_string (gnm_hlink_get_target (link));
+		if (lnk)
+			return value_new_string (gnm_hlink_get_target (lnk));
 	}
 
 	return value_new_empty ();
