@@ -19,8 +19,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <gnumeric-config.h>
 #include <gnumeric.h>
@@ -332,7 +331,7 @@ gnumeric_left (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 /***************************************************************************/
 
 static GnmFuncHelp const help_leftb[] = {
-	{ GNM_FUNC_HELP_NAME, F_("LEFTB:the first characters  of the string @{s} comprising at most @{num_bytes} bytes")},
+	{ GNM_FUNC_HELP_NAME, F_("LEFTB:the first characters of the string @{s} comprising at most @{num_bytes} bytes")},
 	{ GNM_FUNC_HELP_ARG, F_("s:the string")},
 	{ GNM_FUNC_HELP_ARG, F_("num_bytes:the maximum number of bytes to return (defaults to 1)")},
 	{ GNM_FUNC_HELP_NOTE, F_("The semantics of this function is subject to change as various applications implement it.")},
@@ -601,13 +600,13 @@ gnumeric_upper (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 /***************************************************************************/
 
 static GnmFuncHelp const help_concatenate[] = {
-        { GNM_FUNC_HELP_NAME, F_("CONCATENATE:the concatenation of the strings @{s1}, @{s2},\xe2\x80\xa6")},
-        { GNM_FUNC_HELP_ARG, F_("s1:first string")},
-        { GNM_FUNC_HELP_ARG, F_("s2:second string")},
+	{ GNM_FUNC_HELP_NAME, F_("CONCATENATE:the concatenation of the strings @{s1}, @{s2},\xe2\x80\xa6")},
+	{ GNM_FUNC_HELP_ARG, F_("s1:first string")},
+	{ GNM_FUNC_HELP_ARG, F_("s2:second string")},
 	{ GNM_FUNC_HELP_EXCEL, F_("This function is Excel compatible.") },
-        { GNM_FUNC_HELP_EXAMPLES, "=CONCATENATE(\"aa\",\"bb\")" },
-        { GNM_FUNC_HELP_SEEALSO, "LEFT,MID,RIGHT"},
-        { GNM_FUNC_HELP_END}
+	{ GNM_FUNC_HELP_EXAMPLES, "=CONCATENATE(\"aa\",\"bb\")" },
+	{ GNM_FUNC_HELP_SEEALSO, "LEFT,MID,RIGHT"},
+	{ GNM_FUNC_HELP_END}
 };
 
 
@@ -616,10 +615,109 @@ gnumeric_concatenate (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv
 {
 	return string_range_function (argc, argv, ei,
 				      range_concatenate,
+				      NULL,
 				      COLLECT_IGNORE_BLANKS,
 				      GNM_ERROR_VALUE);
 }
 
+static GnmFuncHelp const help_concat[] = {
+	{ GNM_FUNC_HELP_NAME, F_("CONCAT:the concatenation of the strings @{s1}, @{s2},\xe2\x80\xa6")},
+	{ GNM_FUNC_HELP_ARG, F_("s1:first string")},
+	{ GNM_FUNC_HELP_ARG, F_("s2:second string")},
+	{ GNM_FUNC_HELP_EXCEL, F_("This function is Excel compatible.") },
+	{ GNM_FUNC_HELP_EXAMPLES, "=CONCAT(\"aa\",\"bb\")" },
+	{ GNM_FUNC_HELP_NOTE, F_("This function is identical to CONCATENATE") },
+	{ GNM_FUNC_HELP_SEEALSO, "LEFT,MID,RIGHT"},
+	{ GNM_FUNC_HELP_END}
+};
+
+static GnmValue *
+gnumeric_concat (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
+{
+	return gnumeric_concatenate (ei, argc, argv);
+}
+
+/***************************************************************************/
+
+static GnmFuncHelp const help_textjoin[] = {
+	{ GNM_FUNC_HELP_NAME, F_("TEXTJOIN:the concatenation of the strings @{s1}, @{s2},\xe2\x80\xa6 delimited by @{del}")},
+	{ GNM_FUNC_HELP_ARG, F_("del:delimiter")},
+	{ GNM_FUNC_HELP_ARG, F_("blank:ignore blanks")},
+	{ GNM_FUNC_HELP_ARG, F_("s1:first string")},
+	{ GNM_FUNC_HELP_ARG, F_("s2:second string")},
+	{ GNM_FUNC_HELP_EXCEL, F_("This function is Excel compatible.") },
+	{ GNM_FUNC_HELP_EXAMPLES, "=TEXTJOIN(\"::\",FALSE,\"aa\",\"bb\")" },
+	{ GNM_FUNC_HELP_SEEALSO, "CONCATENATE"},
+	{ GNM_FUNC_HELP_END}
+};
+
+struct cb_textjoin {
+	char *delim;
+	gboolean ignore_blanks;
+};
+
+static int
+range_textjoin (GPtrArray *data, char **pres, gpointer user_)
+{
+	struct cb_textjoin *user = user_;
+	GString *res = g_string_new (NULL);
+	gboolean first = TRUE;
+	unsigned ui;
+
+	for (ui = 0; ui < data->len; ui++) {
+		const char *s = g_ptr_array_index (data, ui);
+
+		if (s[0] == 0 && user->ignore_blanks)
+			continue;
+
+		if (first)
+			first = FALSE;
+		else
+			g_string_append (res, user->delim);
+
+		g_string_append (res, s);
+	}
+
+	*pres = g_string_free (res, FALSE);
+	return 0;
+}
+
+static GnmValue *
+gnumeric_textjoin (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
+{
+	GnmValue *v;
+	gboolean err;
+	struct cb_textjoin data;
+
+	data.delim = NULL;
+
+	if (argc < 3)
+		return value_new_error_VALUE (ei->pos);
+
+	v = gnm_expr_eval (argv[0], ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+	if (VALUE_IS_ERROR (v))
+		goto done;
+	data.delim = value_get_as_string (v);
+	value_release (v);
+
+	v = gnm_expr_eval (argv[1], ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+	if (VALUE_IS_ERROR (v))
+		goto done;
+	data.ignore_blanks = value_get_as_bool (v, &err); // What about err?
+	value_release (v);
+
+	v = string_range_function (argc - 2, argv + 2, ei,
+				   range_textjoin,
+				   &data,
+				   data.ignore_blanks ? COLLECT_IGNORE_BLANKS : 0,
+				   GNM_ERROR_VALUE);
+
+done:
+	g_free (data.delim);
+
+	return v;
+}
+
 /***************************************************************************/
 
 static GnmFuncHelp const help_rept[] = {
@@ -1417,7 +1515,7 @@ gnumeric_searchb (GnmFuncEvalInfo *ei, GnmValue const * const *argv)
 static GnmFuncHelp const help_asc[] = {
 	{ GNM_FUNC_HELP_NAME, F_("ASC:text with full-width katakana and ASCII characters converted to half-width")},
 	{ GNM_FUNC_HELP_ARG, F_("text:string")},
-	{ GNM_FUNC_HELP_DESCRIPTION, F_("ASC converts full-width katakana and ASCII characters to half-width equivalent characters, copying all others. ")},
+	{ GNM_FUNC_HELP_DESCRIPTION, F_("ASC converts full-width katakana and ASCII characters to half-width equivalent characters, copying all others.")},
 	{ GNM_FUNC_HELP_DESCRIPTION, F_("The distinction between half-width and full-width characters is described in http://www.unicode.org/reports/tr11/.")},
 	{ GNM_FUNC_HELP_EXCEL, F_("For most strings, this function has the same effect as in Excel.")},
 	{ GNM_FUNC_HELP_NOTE, F_("While in obsolete encodings ASC used to translate between 2-byte and 1-byte characters, this is not the case in UTF-8.")},
@@ -1545,7 +1643,7 @@ static GnmFuncHelp const help_jis[] = {
 	{ GNM_FUNC_HELP_NAME, F_("JIS:text with half-width katakana and ASCII characters converted to full-width")},
 	{ GNM_FUNC_HELP_ARG, F_("text:original text")},
 	{ GNM_FUNC_HELP_DESCRIPTION, F_("JIS converts half-width katakana and ASCII characters "
-					"to full-width equivalent characters, copying all others. ")},
+					"to full-width equivalent characters, copying all others.")},
 	{ GNM_FUNC_HELP_DESCRIPTION, F_("The distinction between half-width and full-width characters "
 					"is described in http://www.unicode.org/reports/tr11/.")},
 	{ GNM_FUNC_HELP_EXCEL, F_("For most strings, this function has the same effect as in Excel.")},
@@ -1673,6 +1771,9 @@ GnmFuncDescriptor const string_functions[] = {
         { "unicode",    "S",                         help_unicode,
 	  gnumeric_unicode, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_BASIC },
+        { "concat", NULL,               help_concat,
+	  NULL, gnumeric_concat, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "concatenate", NULL,               help_concatenate,
 	  NULL, gnumeric_concatenate, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
@@ -1751,6 +1852,9 @@ GnmFuncDescriptor const string_functions[] = {
         { "text",       "Ss",           help_text,
 	  gnumeric_text, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+        { "textjoin", NULL, help_textjoin,
+	  NULL, gnumeric_textjoin, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "trim",       "S",                         help_trim,
 	  gnumeric_trim, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
