@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
   **/
 
 #include <gnumeric-config.h>
@@ -314,7 +313,7 @@ setup_pattern_button (GdkScreen *screen,
 	if (tmp != NULL) {
 		GtkButton *button = GTK_BUTTON (tmp);
 		if (do_image) {
-			char *res = g_strconcat ("/org/gnumeric/gnumeric/images/", name, ".xpm", NULL);
+			char *res = g_strconcat ("/org/gnumeric/gnumeric/images/", name, ".png", NULL);
 			GtkWidget *image;
 			if (from_icon)
 				image = gtk_image_new_from_icon_name (name, GTK_ICON_SIZE_DIALOG);
