@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2001-2006 Jody Goldberg (jody@gnome.org)
  * Copyright (C) 1998-2000 Miguel de Icaza (miguel@gnu.org)
- * Copyright (C) 2000-2009 Morten Welinder (terra@gnome.org)
+ * Copyright (C) 2000-2018 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -506,7 +506,10 @@ gnm_expr_free (GnmExpr const *expr)
 
 	case GNM_EXPR_OP_ARRAY_CORNER:
 		value_release (expr->array_corner.value);
-		gnm_expr_free (expr->array_corner.expr);
+		// A proper corner will not have NULL here, but we explicitly allow it
+		// during construction, so allow it here too.
+		if (expr->array_corner.expr)
+			gnm_expr_free (expr->array_corner.expr);
 		CHUNK_FREE (expression_pool_big, (gpointer)expr);
 		break;
 
@@ -1518,67 +1521,11 @@ gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
 		}
 		return handle_empty (res, flags);
 
-	case GNM_EXPR_OP_ARRAY_CORNER: {
-		GnmEvalPos range_pos = *pos;
-		range_pos.array = &expr->array_corner;
+	case GNM_EXPR_OP_ARRAY_CORNER:
+	case GNM_EXPR_OP_ARRAY_ELEM:
+		g_warning ("Unexpected array expressions encountered");
+		return value_new_error_VALUE (pos);
 
-		a = gnm_expr_eval (expr->array_corner.expr, &range_pos,
-			flags | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
-
-		value_release (expr->array_corner.value);
-
-		/* Store real result (cast away const)*/
-		((GnmExpr*)expr)->array_corner.value = a;
-
-		if (a != NULL &&
-		    (VALUE_IS_CELLRANGE (a) || VALUE_IS_ARRAY (a))) {
-			if (value_area_get_width (a, pos) <= 0 ||
-			    value_area_get_height (a, pos) <= 0)
-				return value_new_error_NA (pos);
-			a = (GnmValue *)value_area_get_x_y (a, 0, 0, pos);
-		}
-		return handle_empty ((a != NULL) ? value_dup (a) : NULL, flags);
-	}
-
-	case GNM_EXPR_OP_ARRAY_ELEM: {
-		/* The upper left corner manages the recalc of the expr */
-		GnmCell *corner = array_elem_get_corner (&expr->array_elem,
-			pos->sheet, &pos->eval);
-		if (!corner ||
-		    !gnm_expr_top_is_array_corner (corner->base.texpr)) {
-			g_warning ("Funky array setup.");
-			return handle_empty (NULL, flags);
-		}
-
-		gnm_cell_eval (corner);
-		a = corner->base.texpr->expr->array_corner.value;
-		if (a == NULL)
-			return handle_empty (NULL, flags);
-
-		if ((VALUE_IS_CELLRANGE (a) || VALUE_IS_ARRAY (a))) {
-			int const num_x = value_area_get_width (a, pos);
-			int const num_y = value_area_get_height (a, pos);
-			int x = expr->array_elem.x;
-			int y = expr->array_elem.y;
-
-			/* Evaluate relative to the upper left corner */
-			GnmEvalPos tmp_ep = *pos;
-			tmp_ep.eval.col -= x;
-			tmp_ep.eval.row -= y;
-
-			/* If the src array is 1 element wide or tall we wrap */
-			if (x >= 1 && num_x == 1)
-				x = 0;
-			if (y >= 1 && num_y == 1)
-				y = 0;
-			if (x >= num_x || y >= num_y)
-				return value_new_error_NA (pos);
-
-			a = (GnmValue *)value_area_get_x_y (a, x, y, &tmp_ep);
-		}
-
-		return handle_empty ((a != NULL) ? value_dup (a) : NULL, flags);
-	}
 	case GNM_EXPR_OP_SET:
 		if (flags & GNM_EXPR_EVAL_PERMIT_NON_SCALAR) {
 			int i;
@@ -1629,16 +1576,12 @@ gnm_expr_simplify_if (GnmExpr const *expr)
 		c = value_get_as_bool (condval, &err);
 		if (err)
 			return NULL;
-	} else if (GNM_EXPR_GET_OPER (cond) == GNM_EXPR_OP_FUNCALL) {
-		if (cond->func.func != gnm_func_lookup ("true", NULL))
-			c = TRUE;
-		else if (cond->func.func != gnm_func_lookup ("false", NULL))
-			c = FALSE;
-		else
-			return NULL;
 	} else
 		return NULL;
 
+	// We used to test for true() and false() as conditions too, but the code
+	// never worked and has been unreachable until now.
+
 	return gnm_expr_copy (expr->func.argv[c ? 1 : 2]);
 }
 
@@ -1892,6 +1835,37 @@ gnm_expr_get_constant (GnmExpr const *expr)
 	return expr->constant.value;
 }
 
+/**
+ * gnm_expr_get_name:
+ * @expr:
+ *
+ * If this expression consists of just a name, return it.
+ */
+GnmNamedExpr const *
+gnm_expr_get_name (GnmExpr const *expr)
+{
+	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_NAME)
+		return NULL;
+
+	return expr->name.name;
+}
+
+
+/**
+ * gnm_expr_get_cellref:
+ * @expr:
+ *
+ * If this expression consists of just a cell reference, return it.
+ */
+GnmCellRef const *
+gnm_expr_get_cellref (GnmExpr const *expr)
+{
+	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_CELLREF)
+		return NULL;
+
+	return &expr->cellref.ref;
+}
+
 
 typedef struct {
 	GnmExprRelocateInfo const *details;
@@ -2272,6 +2246,16 @@ gnm_expr_get_func_def (GnmExpr const *expr)
 	return expr->func.func;
 }
 
+GnmExpr const *
+gnm_expr_get_func_arg (GnmExpr const *expr, int i)
+{
+	g_return_val_if_fail (expr != NULL, NULL);
+	g_return_val_if_fail (GNM_EXPR_GET_OPER (expr) == GNM_EXPR_OP_FUNCALL, NULL);
+	g_return_val_if_fail (i >= 0 && i < expr->func.argc, NULL);
+
+	return expr->func.argv[i];
+}
+
 
 static void
 cellref_boundingbox (GnmCellRef const *cr, Sheet const *sheet, GnmRange *bound)
@@ -2902,7 +2886,7 @@ cb_get_ranges (GnmExpr const *expr, GnmExprWalk *data)
 {
 	GSList **pranges = data->user;
 
-	/* There's no real reason to exclude ranges here, except that
+	/* There's no real reason to exclude names here, except that
 	   we used to do so.  */
 	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_NAME) {
 		GnmValue *v = gnm_expr_get_range (expr);
@@ -3100,28 +3084,109 @@ gnm_expr_top_is_volatile (GnmExprTop const *texpr)
 }
 
 
+static GnmValue *
+gnm_expr_top_eval_array_corner (GnmExprTop const *texpr,
+				GnmEvalPos const *pos,
+				GnmExprEvalFlags flags)
+{
+	GnmExpr const *expr = texpr->expr;
+	GnmEvalPos pos2;
+	GnmValue *a;
+
+	pos2 = *pos;
+	pos2.array_texpr = texpr;
+	a = gnm_expr_eval (expr->array_corner.expr, &pos2,
+				     flags | GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
+
+	value_release (expr->array_corner.value);
+
+	/* Store real result (cast away const)*/
+	((GnmExpr*)expr)->array_corner.value = a;
+
+	if (a != NULL &&
+	    (VALUE_IS_CELLRANGE (a) || VALUE_IS_ARRAY (a))) {
+		if (value_area_get_width (a, pos) <= 0 ||
+		    value_area_get_height (a, pos) <= 0)
+			return value_new_error_NA (pos);
+		a = (GnmValue *)value_area_get_x_y (a, 0, 0, pos);
+	}
+	return handle_empty ((a != NULL) ? value_dup (a) : NULL, flags);
+}
+
+static GnmValue *
+gnm_expr_top_eval_array_elem (GnmExprTop const *texpr,
+			      GnmEvalPos const *pos,
+			      GnmExprEvalFlags flags)
+{
+	GnmExpr const *expr = texpr->expr;
+	/* The upper left corner manages the recalc of the expr */
+	GnmCell *corner = array_elem_get_corner (&expr->array_elem,
+						 pos->sheet, &pos->eval);
+	GnmValue *a;
+
+	if (!corner ||
+	    !gnm_expr_top_is_array_corner (corner->base.texpr)) {
+		g_warning ("Funky array setup.");
+		return handle_empty (NULL, flags);
+	}
+
+	gnm_cell_eval (corner);
+	a = gnm_expr_top_get_array_value (corner->base.texpr);
+	if (a == NULL)
+		return handle_empty (NULL, flags);
+
+	if ((VALUE_IS_CELLRANGE (a) || VALUE_IS_ARRAY (a))) {
+		int const num_x = value_area_get_width (a, pos);
+		int const num_y = value_area_get_height (a, pos);
+		int x = expr->array_elem.x;
+		int y = expr->array_elem.y;
+
+		/* Evaluate relative to the upper left corner */
+		GnmEvalPos tmp_ep = *pos;
+		tmp_ep.eval.col -= x;
+		tmp_ep.eval.row -= y;
+
+		/* If the src array is 1 element wide or tall we wrap */
+		if (x >= 1 && num_x == 1)
+			x = 0;
+		if (y >= 1 && num_y == 1)
+			y = 0;
+		if (x >= num_x || y >= num_y)
+			return value_new_error_NA (pos);
+
+		a = (GnmValue *)value_area_get_x_y (a, x, y, &tmp_ep);
+	}
+
+	return handle_empty ((a != NULL) ? value_dup (a) : NULL, flags);
+}
+
 GnmValue *
 gnm_expr_top_eval (GnmExprTop const *texpr,
 		   GnmEvalPos const *pos,
 		   GnmExprEvalFlags flags)
 {
 	GnmValue *res;
-	GnmEvalPos ep;
-	GnmExprArrayCorner array;
 
 	g_return_val_if_fail (GNM_IS_EXPR_TOP (texpr), NULL);
 
 	gnm_app_recalc_start ();
-	if ((flags & GNM_EXPR_EVAL_ARRAY_CONTEXT) && !pos->array) {
-		array.oper = GNM_EXPR_OP_ARRAY_CORNER;
-		array.cols = array.rows = 1;
-		array.value = NULL;
-		array.expr = texpr->expr;
-		ep = *pos;
-		ep.array = &array;
-		pos = &ep;
-	}
-	res = gnm_expr_eval (texpr->expr, pos, flags);
+
+	if (gnm_expr_top_is_array_corner (texpr))
+		res = gnm_expr_top_eval_array_corner (texpr, pos, flags);
+	else if (gnm_expr_top_is_array_elem (texpr, NULL, NULL))
+		res = gnm_expr_top_eval_array_elem (texpr, pos, flags);
+	else if ((flags & GNM_EXPR_EVAL_ARRAY_CONTEXT) && !eval_pos_is_array_context (pos)) {
+		// Array context out of the blue.  Fake it.
+		GnmEvalPos pos2;
+		GnmExprTop const *fake = gnm_expr_top_new_array_corner (1, 1, NULL);
+		((GnmExpr *)(fake->expr))->array_corner.expr = texpr->expr; // Patch in our expr
+		pos2 = *pos;
+		pos2.array_texpr = fake;
+		res = gnm_expr_eval (texpr->expr, &pos2, flags);
+		((GnmExpr *)(fake->expr))->array_corner.expr = NULL;
+		gnm_expr_top_unref (fake);
+	} else
+		res = gnm_expr_eval (texpr->expr, pos, flags);
 	gnm_app_recalc_finish ();
 
 	return res;
@@ -3207,11 +3272,7 @@ GnmCellRef const *
 gnm_expr_top_get_cellref (GnmExprTop const *texpr)
 {
 	g_return_val_if_fail (GNM_IS_EXPR_TOP (texpr), NULL);
-
-	if (GNM_EXPR_GET_OPER (texpr->expr) != GNM_EXPR_OP_CELLREF)
-		return NULL;
-
-	return &texpr->expr->cellref.ref;
+	return gnm_expr_get_cellref (texpr->expr);
 }
 
 static GnmExpr const *
@@ -3301,21 +3362,6 @@ gnm_expr_top_is_rangeref (GnmExprTop const *texpr)
 	return gnm_expr_is_rangeref (texpr->expr);
 }
 
-/**
- * gnm_expr_top_new_array_corner:
- * @expr: #GnmExprTop
- *
- * Returns: (transfer none): the #GnmExprArrayCorner or %NULL.
- **/
-GnmExprArrayCorner const *
-gnm_expr_top_get_array_corner (GnmExprTop const *texpr)
-{
-	g_return_val_if_fail (GNM_IS_EXPR_TOP (texpr), NULL);
-	return GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER
-		? &texpr->expr->array_corner
-		: NULL;
-}
-
 gboolean
 gnm_expr_top_is_array_corner (GnmExprTop const *texpr)
 {
@@ -3323,6 +3369,34 @@ gnm_expr_top_is_array_corner (GnmExprTop const *texpr)
 	return GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER;
 }
 
+void
+gnm_expr_top_get_array_size (GnmExprTop const *texpr, int *cols, int *rows)
+{
+	g_return_if_fail (GNM_IS_EXPR_TOP (texpr));
+	g_return_if_fail (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER);
+
+	if (cols)
+		*cols = texpr->expr->array_corner.cols;
+	if (rows)
+		*rows = texpr->expr->array_corner.rows;
+}
+
+GnmValue *
+gnm_expr_top_get_array_value (GnmExprTop const *texpr)
+{
+	g_return_val_if_fail (GNM_IS_EXPR_TOP (texpr), NULL);
+	g_return_val_if_fail (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER, NULL);
+	return texpr->expr->array_corner.value;
+}
+
+GnmExpr const *
+gnm_expr_top_get_array_expr (GnmExprTop const *texpr)
+{
+	g_return_val_if_fail (GNM_IS_EXPR_TOP (texpr), NULL);
+	g_return_val_if_fail (GNM_EXPR_GET_OPER (texpr->expr) == GNM_EXPR_OP_ARRAY_CORNER, NULL);
+	return texpr->expr->array_corner.expr;
+}
+
 gboolean
 gnm_expr_top_is_array_elem (GnmExprTop const *texpr, int *x, int *y)
 {
