@@ -797,7 +797,10 @@ random_geometric (gnm_float p)
 		u = random_01 ();
 	} while (u == 0);
 
-	return gnm_floor (gnm_log (u) / gnm_log1p (-p) + 1);
+	/*
+	 * Change from gsl version: we have support {0,1,2,...}
+	 */
+	return gnm_floor (gnm_log (u) / gnm_log1p (-p));
 }
 
 gnm_float
@@ -830,7 +833,7 @@ random_logarithmic (gnm_float p)
 		do {
 			u = random_01 ();
 		} while (u == 0);
-		q = gnm_expm1 (c * u);
+		q = -gnm_expm1 (c * u);
 
 		if (v <= q * q)
 			return gnm_floor (1 + gnm_log (v) / gnm_log (q));
