@@ -30,12 +30,14 @@
 #include "xml-sax.h"
 #include "hlink.h"
 #include "input-msg.h"
+#include "expr-name.h"
+#include "workbook-priv.h"
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-output-stdio.h>
 #include <gsf/gsf-input.h>
 
-/* FIXME: Namespace?  */
-#define DIFF "ssdiff:"
+#define DIFF "s:"
+#define SSDIFF_DTD "http://www.gnumeric.org/ssdiff.dtd" // No such file yet
 
 static gboolean ssdiff_show_version = FALSE;
 static gboolean ssdiff_highlight = FALSE;
@@ -81,50 +83,67 @@ static const GOptionEntry ssdiff_options [] = {
 typedef struct GnmDiffState_ GnmDiffState;
 
 typedef struct {
-	/* Start comparison of two workbooks.  */
+	// Start comparison of two workbooks.
 	gboolean (*diff_start) (GnmDiffState *state);
 
-	/* Finish comparison started with above.  */
+	// Finish comparison started with above.
 	void (*diff_end) (GnmDiffState *state);
 
+	// Clean up allocations
+	void (*dtor) (GnmDiffState *state);
+
 	/* ------------------------------ */
 
-	/* Start looking at a sheet.  Either sheet might be NULL.  */
+	// Start looking at a sheet.  Either sheet might be NULL.
 	void (*sheet_start) (GnmDiffState *state,
 			     Sheet const *os, Sheet const *ns);
 
-	/* Finish sheet started with above.  */
+	// Finish sheet started with above.
 	void (*sheet_end) (GnmDiffState *state);
 
-	/* The order of sheets has changed.  */
+	// The order of sheets has changed.
 	void (*sheet_order_changed) (GnmDiffState *state);
 
-	/* An integer attribute of the sheet has changed.  */
+	// An integer attribute of the sheet has changed.
 	void (*sheet_attr_int_changed) (GnmDiffState *state, const char *name,
 					int o, int n);
 
 	/* ------------------------------ */
 
-	/* A cell was changed/added/removed.  */
+	// Column or Row information changed
+	void (*colrow_changed) (GnmDiffState *state,
+				ColRowInfo const *oc, ColRowInfo const *nc,
+				gboolean is_cols, int i);
+
+	/* ------------------------------ */
+
+	// A cell was changed/added/removed.
 	void (*cell_changed) (GnmDiffState *state,
 			      GnmCell const *oc, GnmCell const *nc);
 
 	/* ------------------------------ */
 
-	/* The style of an area was changed.  */
+	// The style of an area was changed.
 	void (*style_changed) (GnmDiffState *state, GnmRange const *r,
-			       Sheet const *osh, Sheet const *nsh,
 			       GnmStyle const *os, GnmStyle const *ns);
+
+	/* ------------------------------ */
+
+	// A defined name was changed
+	void (*name_changed) (GnmDiffState *state,
+			      GnmNamedExpr const *on, GnmNamedExpr const *nn);
 } GnmDiffActions;
 
+typedef struct {
+	char *url;
+	GsfInput *input;
+	Workbook *wb;
+	WorkbookView *wbv;
+}  GnmDiffStateFile;
+
 struct GnmDiffState_ {
 	GOIOContext *ioc;
-	struct GnmDiffStateFile_ {
-		char *url;
-		GsfInput *input;
-		Workbook *wb;
-		WorkbookView *wbv;
-	} old, new;
+	GnmDiffStateFile old, new;
 
 	const GnmDiffActions *actions;
 
@@ -132,14 +151,17 @@ struct GnmDiffState_ {
 
 	GsfOutput *output;
 
-	/* The following for xml mode.  */
-	GsfXMLOut *xml;
-	gboolean cells_open;
-	gboolean styles_open;
-	GnmConventions *convs;
+	// Valid when comparing sheets
+	Sheet *old_sheet, *new_sheet;
+	GnmRange common_range;
 
-	/* The following for highlight mode.  */
-	struct GnmDiffStateFile_ highlight;
+	// The following for xml mode.
+	GsfXMLOut *xml;
+	const char *xml_section;
+	GnmConventions *xml_convs;
+
+	// The following for highlight mode.
+	GnmDiffStateFile highlight;
 	GOFileSaver const *highlight_fs;
 	GnmStyle *highlight_style;
 };
@@ -155,6 +177,11 @@ null_diff_end (G_GNUC_UNUSED GnmDiffState *state)
 {
 }
 
+static void
+null_dtor (G_GNUC_UNUSED GnmDiffState *state)
+{
+}
+
 static void
 null_sheet_start (G_GNUC_UNUSED GnmDiffState *state,
 		  G_GNUC_UNUSED Sheet const *os,
@@ -180,11 +207,23 @@ null_sheet_attr_int_changed (G_GNUC_UNUSED GnmDiffState *state,
 {
 }
 
+static void
+null_colrow_changed (G_GNUC_UNUSED GnmDiffState *state,
+		     G_GNUC_UNUSED ColRowInfo const *oc, G_GNUC_UNUSED ColRowInfo const *nc,
+		     G_GNUC_UNUSED gboolean is_cols, G_GNUC_UNUSED int i)
+{
+}
+
+static void
+null_name_changed (G_GNUC_UNUSED GnmDiffState *state,
+		   G_GNUC_UNUSED GnmNamedExpr const *on, G_GNUC_UNUSED GnmNamedExpr const *nn)
+{
+}
+
 /* -------------------------------------------------------------------------- */
 
 static gboolean
-read_file (struct GnmDiffStateFile_ *dsf, const char *filename,
-	   GOIOContext *ioc)
+read_file (GnmDiffStateFile *dsf, const char *filename, GOIOContext *ioc)
 {
 	GError *err = NULL;
 
@@ -214,9 +253,10 @@ read_file (struct GnmDiffStateFile_ *dsf, const char *filename,
 }
 
 static void
-clear_file_state (struct GnmDiffStateFile_ *dsf)
+clear_file_state (GnmDiffStateFile *dsf)
 {
 	g_free (dsf->url);
+	dsf->url = NULL;
 	g_clear_object (&dsf->wb);
 	g_clear_object (&dsf->input);
 }
@@ -264,6 +304,18 @@ def_sheet_attr_int_changed (GnmDiffState *state, const char *name,
 			   name);
 }
 
+static void
+def_colrow_changed (GnmDiffState *state, ColRowInfo const *oc, ColRowInfo const *nc,
+		    gboolean is_cols, int i)
+{
+	if (is_cols)
+		gsf_output_printf (state->output, _("Column %s changed.\n"),
+				   col_name (i));
+	else
+		gsf_output_printf (state->output, _("Row %d changed.\n"),
+				   i + 1);
+}
+
 static void
 def_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 {
@@ -279,8 +331,6 @@ def_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 
 static void
 def_style_changed (GnmDiffState *state, GnmRange const *r,
-		   G_GNUC_UNUSED Sheet const *osh,
-		   G_GNUC_UNUSED Sheet const *nsh,
 		   G_GNUC_UNUSED GnmStyle const *os,
 		   G_GNUC_UNUSED GnmStyle const *ns)
 {
@@ -288,15 +338,32 @@ def_style_changed (GnmDiffState *state, GnmRange const *r,
 			   range_as_string (r));
 }
 
+static void
+def_name_changed (GnmDiffState *state,
+		  GnmNamedExpr const *on, GnmNamedExpr const *nn)
+{
+	if (on && nn)
+		gsf_output_printf (state->output, _("Name %s changed.\n"), expr_name_name (on));
+	else if (on)
+		gsf_output_printf (state->output, _("Name %s removed.\n"), expr_name_name (on));
+	else if (nn)
+		gsf_output_printf (state->output, _("Name %s added.\n"), expr_name_name (nn));
+	else
+		g_assert_not_reached ();
+}
+
 static const GnmDiffActions default_actions = {
 	null_diff_start,
 	null_diff_end,
+	null_dtor,
 	def_sheet_start,
 	null_sheet_end,
 	def_sheet_order_changed,
 	def_sheet_attr_int_changed,
+	def_colrow_changed,
 	def_cell_changed,
 	def_style_changed,
+	def_name_changed,
 };
 
 /* -------------------------------------------------------------------------- */
@@ -304,10 +371,16 @@ static const GnmDiffActions default_actions = {
 static gboolean
 xml_diff_start (GnmDiffState *state)
 {
+	char *attr;
+
 	state->xml = gsf_xml_out_new (state->output);
-	state->convs = gnm_xml_io_conventions ();
+	state->xml_convs = gnm_xml_io_conventions ();
 
 	gsf_xml_out_start_element (state->xml, DIFF "Diff");
+	attr = g_strdup ("xmlns:" DIFF);
+	attr[strlen (attr) - 1] = 0;
+	gsf_xml_out_add_cstr (state->xml, attr, SSDIFF_DTD);
+	g_free (attr);
 
 	return FALSE;
 }
@@ -318,11 +391,45 @@ xml_diff_end (GnmDiffState *state)
 	gsf_xml_out_end_element (state->xml); /* </Diff> */
 }
 
+static void
+xml_dtor (GnmDiffState *state)
+{
+	g_clear_object (&state->xml);
+
+	if (state->xml_convs) {
+		gnm_conventions_unref (state->xml_convs);
+		state->xml_convs = NULL;
+	}
+}
+
+static void
+xml_close_section (GnmDiffState *state)
+{
+	if (state->xml_section) {
+		gsf_xml_out_end_element (state->xml);
+		state->xml_section = NULL;
+	}
+}
+
+static void
+xml_open_section (GnmDiffState *state, const char *section)
+{
+	if (state->xml_section && g_str_equal (section, state->xml_section))
+		return;
+
+	xml_close_section (state);
+	gsf_xml_out_start_element (state->xml, section);
+	state->xml_section = section;
+}
+
 static void
 xml_sheet_start (GnmDiffState *state, Sheet const *os, Sheet const *ns)
 {
 	Sheet const *sheet = os ? os : ns;
 
+	// We might have an open section for global names
+	xml_close_section (state);
+
 	gsf_xml_out_start_element (state->xml, DIFF "Sheet");
 	gsf_xml_out_add_cstr (state->xml, "Name", sheet->name_unquoted);
 	if (os)
@@ -331,29 +438,10 @@ xml_sheet_start (GnmDiffState *state, Sheet const *os, Sheet const *ns)
 		gsf_xml_out_add_int (state->xml, "New", ns->index_in_wb);
 }
 
-static void
-xml_close_cells (GnmDiffState *state)
-{
-	if (state->cells_open) {
-		gsf_xml_out_end_element (state->xml); /* </Cells> */
-		state->cells_open = FALSE;
-	}
-}
-
-static void
-xml_close_styles (GnmDiffState *state)
-{
-	if (state->styles_open) {
-		gsf_xml_out_end_element (state->xml); /* </Styles> */
-		state->styles_open = FALSE;
-	}
-}
-
 static void
 xml_sheet_end (GnmDiffState *state)
 {
-	xml_close_cells (state);
-	xml_close_styles (state);
+	xml_close_section (state);
 	gsf_xml_out_end_element (state->xml); /* </Sheet> */
 }
 
@@ -372,35 +460,77 @@ xml_sheet_attr_int_changed (GnmDiffState *state, const char *name,
 }
 
 static void
-output_cell (GnmDiffState *state, GnmCell const *cell,
-	     const char *tag, const char *valtag, const char *fmttag)
+xml_output_texpr (GnmDiffState *state, GnmExprTop const *texpr, GnmParsePos const *pos,
+		  const char *tag)
 {
+	GnmConventionsOut out;
 	GString *str;
 
+	out.accum = str = g_string_sized_new (100);
+	out.pp    = pos;
+	out.convs = state->xml_convs;
+
+	g_string_append_c (str, '=');
+	gnm_expr_top_as_gstring (texpr, &out);
+
+	gsf_xml_out_add_cstr (state->xml, tag, str->str);
+	g_string_free (str, TRUE);
+}
+
+static void
+xml_output_cell (GnmDiffState *state, GnmCell const *cell,
+		 const char *tag, const char *valtag, const char *fmttag)
+{
 	if (!cell)
 		return;
 
-	str = g_string_sized_new (100);
 	if (gnm_cell_has_expr (cell)) {
-		GnmConventionsOut out;
 		GnmParsePos pp;
-
-		out.accum = str;
-		out.pp    = parse_pos_init_cell (&pp, cell);
-		out.convs = state->convs;
-
-		g_string_append_c (str, '=');
-		gnm_expr_top_as_gstring (cell->base.texpr, &out);
+		parse_pos_init_cell (&pp, cell);
+		xml_output_texpr (state, cell->base.texpr, &pp, tag);
 	} else {
 		GnmValue const *v = cell->value;
-		value_get_as_gstring (v, str, state->convs);
+		GString *str = g_string_sized_new (100);
+		value_get_as_gstring (v, str, state->xml_convs);
+		gsf_xml_out_add_cstr (state->xml, tag, str->str);
+		g_string_free (str, TRUE);
 		gsf_xml_out_add_int (state->xml, valtag, v->v_any.type);
 		if (VALUE_FMT (v))
 			gsf_xml_out_add_cstr (state->xml, fmttag, go_format_as_XL (VALUE_FMT (v)));
 	}
+}
 
-	gsf_xml_out_add_cstr (state->xml, tag, str->str);
-	g_string_free (str, TRUE);
+static void
+xml_colrow_changed (GnmDiffState *state, ColRowInfo const *oc, ColRowInfo const *nc,
+		    gboolean is_cols, int i)
+{
+	xml_open_section (state, is_cols ? DIFF "Cols" : DIFF "Rows");
+
+	gsf_xml_out_start_element (state->xml, is_cols ? DIFF "ColInfo" : DIFF "RowInfo");
+	if (i >= 0) gsf_xml_out_add_int (state->xml, "No", i);
+
+	if (oc->size_pts != nc->size_pts) {
+		gsf_xml_out_add_float (state->xml, "OldUnit", oc->size_pts, 4);
+		gsf_xml_out_add_float (state->xml, "NewUnit", nc->size_pts, 4);
+	}
+	if (oc->hard_size != nc->hard_size) {
+		gsf_xml_out_add_bool (state->xml, "OldHardSize", oc->hard_size);
+		gsf_xml_out_add_bool (state->xml, "NewHardSize", nc->hard_size);
+	}
+	if (oc->visible != nc->visible) {
+		gsf_xml_out_add_bool (state->xml, "OldHidden", !oc->visible);
+		gsf_xml_out_add_bool (state->xml, "NewHidden", !nc->visible);
+	}
+	if (oc->is_collapsed != nc->is_collapsed) {
+		gsf_xml_out_add_bool (state->xml, "OldCollapsed", oc->is_collapsed);
+		gsf_xml_out_add_bool (state->xml, "NewCollapsed", nc->is_collapsed);
+	}
+	if (oc->outline_level != nc->outline_level) {
+		gsf_xml_out_add_int (state->xml, "OldOutlineLevel", oc->outline_level);
+		gsf_xml_out_add_int (state->xml, "NewOutlineLevel", nc->outline_level);
+	}
+
+	gsf_xml_out_end_element (state->xml); /* </ColInfo> or </RowInfo> */
 }
 
 static void
@@ -408,10 +538,7 @@ xml_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 {
 	const GnmCellPos *pos;
 
-	if (!state->cells_open) {
-		gsf_xml_out_start_element (state->xml, DIFF "Cells");
-		state->cells_open = TRUE;
-	}
+	xml_open_section (state, DIFF "Cells");
 
 	gsf_xml_out_start_element (state->xml, DIFF "Cell");
 
@@ -419,15 +546,15 @@ xml_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 	gsf_xml_out_add_int (state->xml, "Row", pos->row);
 	gsf_xml_out_add_int (state->xml, "Col", pos->col);
 
-	output_cell (state, oc, "Old", "OldValueType", "OldValueFormat");
-	output_cell (state, nc, "New", "NewValueType", "NewValueFormat");
+	xml_output_cell (state, oc, "Old", "OldValueType", "OldValueFormat");
+	xml_output_cell (state, nc, "New", "NewValueType", "NewValueFormat");
 
 	gsf_xml_out_end_element (state->xml); /* </Cell> */
 }
 
 #define DO_INT(what,fun)					\
   do {								\
-	  gsf_xml_out_start_element (state->xml, (what));	\
+	  gsf_xml_out_start_element (state->xml, DIFF what);	\
 	  gsf_xml_out_add_int (state->xml, "Old", (fun) (os));	\
 	  gsf_xml_out_add_int (state->xml, "New", (fun) (ns));	\
 	  gsf_xml_out_end_element (state->xml);			\
@@ -435,10 +562,10 @@ xml_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 
 #define DO_INTS(what,fun,oobj,nobj)					\
   do {									\
-  	  int oi = (oobj) ? (fun) (oobj) : 0;			\
-	  int ni = (nobj) ? (fun) (nobj) : 0;			\
+	  int oi = (oobj) ? (fun) (oobj) : 0;				\
+	  int ni = (nobj) ? (fun) (nobj) : 0;				\
 	  if (oi != ni || !(oobj) != !(nobj)) {				\
-		  gsf_xml_out_start_element (state->xml, (what));	\
+		  gsf_xml_out_start_element (state->xml, DIFF what);	\
 		  if (oobj) gsf_xml_out_add_int (state->xml, "Old", oi); \
 		  if (nobj) gsf_xml_out_add_int (state->xml, "New", ni); \
 		  gsf_xml_out_end_element (state->xml);			\
@@ -450,7 +577,7 @@ xml_cell_changed (GnmDiffState *state, GnmCell const *oc, GnmCell const *nc)
 	  const char *ostr = (oobj) ? (fun) (oobj) : NULL;		\
 	  const char *nstr = (nobj) ? (fun) (nobj) : NULL;		\
 	  if (g_strcmp0 (ostr, nstr)) {					\
-		  gsf_xml_out_start_element (state->xml, (what));	\
+		  gsf_xml_out_start_element (state->xml, DIFF what);	\
 		  if (ostr) gsf_xml_out_add_cstr (state->xml, "Old", ostr); \
 		  if (nstr) gsf_xml_out_add_cstr (state->xml, "New", nstr); \
 		  gsf_xml_out_end_element (state->xml);			\
@@ -483,19 +610,12 @@ cb_validation_use_dropdown (GnmValidation const *v)
 
 static void
 xml_style_changed (GnmDiffState *state, GnmRange const *r,
-		   G_GNUC_UNUSED Sheet const *osh,
-		   G_GNUC_UNUSED Sheet const *nsh,
 		   GnmStyle const *os, GnmStyle const *ns)
 {
 	unsigned int conflicts;
 	GnmStyleElement e;
 
-	xml_close_cells (state);
-
-	if (!state->styles_open) {
-		gsf_xml_out_start_element (state->xml, DIFF "Styles");
-		state->styles_open = TRUE;
-	}
+	xml_open_section (state, DIFF "Styles");
 
 	gsf_xml_out_start_element (state->xml, DIFF "StyleRegion");
 	gsf_xml_out_add_uint (state->xml, "startCol", r->start.col);
@@ -512,7 +632,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 			GnmColor *oc = gnm_style_get_back_color (os);
 			GnmColor *nc = gnm_style_get_back_color (ns);
 
-			gsf_xml_out_start_element (state->xml, "BackColor");
+			gsf_xml_out_start_element (state->xml, DIFF "BackColor");
 			gnm_xml_out_add_gocolor (state->xml, "Old", oc->go_color);
 			gnm_xml_out_add_gocolor (state->xml, "New", nc->go_color);
 			if (oc->is_auto != nc->is_auto) {
@@ -524,7 +644,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 		}
 
 		case MSTYLE_COLOR_PATTERN:
-			gsf_xml_out_start_element (state->xml, "PatternColor");
+			gsf_xml_out_start_element (state->xml, DIFF "PatternColor");
 			gnm_xml_out_add_gocolor (state->xml, "Old", gnm_style_get_pattern_color (os)->go_color);
 			gnm_xml_out_add_gocolor (state->xml, "New", gnm_style_get_pattern_color (ns)->go_color);
 			gsf_xml_out_end_element (state->xml);
@@ -545,7 +665,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 				"Diagonal"
 			};
 
-			char *tag = g_strconcat ("Border",
+			char *tag = g_strconcat (DIFF "Border",
 						 border_names[e - MSTYLE_BORDER_TOP],
 						 NULL);
 			GnmBorder const *ob = gnm_style_get_border (os, e);
@@ -567,14 +687,14 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 			break;
 
 		case MSTYLE_FONT_COLOR:
-			gsf_xml_out_start_element (state->xml, "FontColor");
+			gsf_xml_out_start_element (state->xml, DIFF "FontColor");
 			gnm_xml_out_add_gocolor (state->xml, "Old", gnm_style_get_font_color (os)->go_color);
 			gnm_xml_out_add_gocolor (state->xml, "New", gnm_style_get_font_color (ns)->go_color);
 			gsf_xml_out_end_element (state->xml);
 			break;
 
 		case MSTYLE_FONT_NAME:
-			gsf_xml_out_start_element (state->xml, "FontName");
+			gsf_xml_out_start_element (state->xml, DIFF "FontName");
 			gsf_xml_out_add_cstr (state->xml, "Old", gnm_style_get_font_name (os));
 			gsf_xml_out_add_cstr (state->xml, "New", gnm_style_get_font_name (ns));
 			gsf_xml_out_end_element (state->xml);
@@ -601,14 +721,14 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 			break;
 
 		case MSTYLE_FONT_SIZE:
-			gsf_xml_out_start_element (state->xml, "FontSize");
+			gsf_xml_out_start_element (state->xml, DIFF "FontSize");
 			gsf_xml_out_add_float (state->xml, "Old", gnm_style_get_font_size (os), 4);
 			gsf_xml_out_add_float (state->xml, "New", gnm_style_get_font_size (ns), 4);
 			gsf_xml_out_end_element (state->xml);
 			break;
 
 		case MSTYLE_FORMAT:
-			gsf_xml_out_start_element (state->xml, "Format");
+			gsf_xml_out_start_element (state->xml, DIFF "Format");
 			gsf_xml_out_add_cstr (state->xml, "Old", go_format_as_XL (gnm_style_get_format (os)));
 			gsf_xml_out_add_cstr (state->xml, "New", go_format_as_XL (gnm_style_get_format (ns)));
 			gsf_xml_out_end_element (state->xml);
@@ -654,7 +774,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 			GnmHLink const *ol = gnm_style_get_hlink (os);
 			GnmHLink const *nl = gnm_style_get_hlink (ns);
 
-			gsf_xml_out_start_element (state->xml, "HLink");
+			gsf_xml_out_start_element (state->xml, DIFF "HLink");
 			if (ol) {
 				gsf_xml_out_add_cstr (state->xml, "OldTarget", gnm_hlink_get_target (ol));
 				gsf_xml_out_add_cstr (state->xml, "OldTip", gnm_hlink_get_tip (ol));
@@ -671,7 +791,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 		case MSTYLE_VALIDATION: {
 			GnmValidation const *ov = gnm_style_get_validation (os);
 			GnmValidation const *nv = gnm_style_get_validation (ns);
-			gsf_xml_out_start_element (state->xml, "Validation");
+			gsf_xml_out_start_element (state->xml, DIFF "Validation");
 			DO_STRINGS ("Message", cb_validation_message, ov, nv);
 			DO_STRINGS ("Title", cb_validation_title, ov, nv);
 			DO_INTS ("AllowBlank", cb_validation_allow_blank, ov, nv);
@@ -684,7 +804,7 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 			GnmInputMsg const *om = gnm_style_get_input_msg (os);
 			GnmInputMsg const *nm = gnm_style_get_input_msg (ns);
 
-			gsf_xml_out_start_element (state->xml, "InputMessage");
+			gsf_xml_out_start_element (state->xml, DIFF "InputMessage");
 			DO_STRINGS ("Message", gnm_input_msg_get_msg, om, nm);
 			DO_STRINGS ("Title", gnm_input_msg_get_title, om, nm);
 			gsf_xml_out_end_element (state->xml); /* </InputMessage> */
@@ -692,12 +812,12 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 		}
 
 		case MSTYLE_CONDITIONS:
-			gsf_xml_out_start_element (state->xml, "Conditions");
+			gsf_xml_out_start_element (state->xml, DIFF "Conditions");
 			gsf_xml_out_end_element (state->xml); /* </Conditions> */
 			break;
 
 		default:
-			gsf_xml_out_start_element (state->xml, "Other");
+			gsf_xml_out_start_element (state->xml, DIFF "Other");
 			gsf_xml_out_end_element (state->xml); /* </Other> */
 			break;
 		}
@@ -707,16 +827,36 @@ xml_style_changed (GnmDiffState *state, GnmRange const *r,
 }
 
 #undef DO_INT
+#undef DO_INTS
+#undef DO_STRINGS
+
+static void
+xml_name_changed (GnmDiffState *state,
+		  GnmNamedExpr const *on, GnmNamedExpr const *nn)
+{
+	xml_open_section (state, DIFF "Names");
+
+	gsf_xml_out_start_element (state->xml, DIFF "Name");
+	gsf_xml_out_add_cstr (state->xml, "Name", expr_name_name (on ? on : nn));
+	if (on)
+		xml_output_texpr (state, on->texpr, &on->pos, "Old");
+	if (nn)
+		xml_output_texpr (state, nn->texpr, &nn->pos, "New");
+	gsf_xml_out_end_element (state->xml); /* </Name> */
+}
 
 static const GnmDiffActions xml_actions = {
 	xml_diff_start,
 	xml_diff_end,
+	xml_dtor,
 	xml_sheet_start,
 	xml_sheet_end,
 	null_sheet_order_changed,
 	xml_sheet_attr_int_changed,
+	xml_colrow_changed,
 	xml_cell_changed,
 	xml_style_changed,
+	xml_name_changed,
 };
 
 /* -------------------------------------------------------------------------- */
@@ -726,22 +866,22 @@ highlight_diff_start (GnmDiffState *state)
 {
 	const char *dst = state->new.url;
 
-	state->highlight_fs = go_file_saver_for_file_name (dst);
+	state->highlight_fs = go_file_saver_for_file_name (ssdiff_output);
 	if (!state->highlight_fs) {
 		g_printerr (_("%s: Unable to guess exporter to use for %s.\n"),
 			    g_get_prgname (),
-			    dst);
+			    ssdiff_output);
 
 		return TRUE;
 	}
 
-	/* We need a copy of one of the files.  Rereading is easy.  */
+	// We need a copy of one of the files.  Rereading is easy.
 	g_object_ref ((state->highlight.input = state->new.input));
 	gsf_input_seek (state->highlight.input, 0, G_SEEK_SET);
 	if (read_file (&state->highlight, dst, state->ioc))
 		return TRUE;
 
-	/* We apply a solid #F3F315 to changed cells.  */
+	// We apply a solid #F3F315 to changed cells.
 	state->highlight_style = gnm_style_new ();
 	gnm_style_set_back_color (state->highlight_style,
 				  gnm_color_new_rgb8 (0xf3, 0xf3, 0x15));
@@ -758,13 +898,22 @@ highlight_diff_end (GnmDiffState *state)
 }
 
 static void
-highlight_apply (GnmDiffState *state, const char *sheetname,
-		 const GnmRange *r)
+highlight_dtor (GnmDiffState *state)
 {
-	Sheet *sheet = workbook_sheet_by_name (state->highlight.wb,
-					       sheetname);
-	if (!sheet)
-		return;
+	clear_file_state (&state->highlight);
+	if (state->highlight_style) {
+		gnm_style_unref (state->highlight_style);
+		state->highlight_style = NULL;
+	}
+}
+
+static void
+highlight_apply (GnmDiffState *state, const GnmRange *r)
+{
+	// We want the highlight sheet corresponding to new_sheet.
+	Sheet *sheet = workbook_sheet_by_index (state->highlight.wb,
+						state->new_sheet->index_in_wb);
+	g_return_if_fail (IS_SHEET (sheet));
 
 	gnm_style_ref (state->highlight_style);
 	sheet_style_apply_range (sheet, r, state->highlight_style);
@@ -775,35 +924,59 @@ highlight_cell_changed (GnmDiffState *state,
 			GnmCell const *oc, GnmCell const *nc)
 {
 	GnmRange r;
-	r.start = nc->pos;
-	r.end = nc->pos;
-	highlight_apply (state, nc->base.sheet->name_unquoted, &r);
+	highlight_apply (state, range_init_cellpos (&r, &(nc ? nc : oc)->pos));
 }
 
 static void
 highlight_style_changed (GnmDiffState *state, GnmRange const *r,
-			 G_GNUC_UNUSED Sheet const *osh,
-			 Sheet const *nsh,
 			 G_GNUC_UNUSED GnmStyle const *os,
 			 G_GNUC_UNUSED GnmStyle const *ns)
 {
-	highlight_apply (state, nsh->name_unquoted, r);
+	highlight_apply (state, r);
 }
 
 
 static const GnmDiffActions highlight_actions = {
 	highlight_diff_start,
 	highlight_diff_end,
+	highlight_dtor,
 	null_sheet_start,
 	null_sheet_end,
 	null_sheet_order_changed,
 	null_sheet_attr_int_changed,
+	null_colrow_changed,
 	highlight_cell_changed,
 	highlight_style_changed,
+	null_name_changed,
 };
 
 /* -------------------------------------------------------------------------- */
 
+static gboolean
+compare_texpr_equal (GnmExprTop const *oe, GnmParsePos const *opp,
+		     GnmExprTop const *ne, GnmParsePos const *npp,
+		     GnmConventions const *convs)
+{
+	char *so, *sn;
+	gboolean eq;
+
+	if (gnm_expr_top_equal (oe, ne))
+		return TRUE;
+
+	// Not equal, but with references to sheets, that is not
+	// necessary.  Compare as strings.
+
+	so = gnm_expr_top_as_string (oe, opp, convs);
+	sn = gnm_expr_top_as_string (ne, npp, convs);
+
+	eq = g_strcmp0 (so, sn) == 0;
+
+	g_free (so);
+	g_free (sn);
+
+	return eq;
+}
+
 static gboolean
 compare_corresponding_cells (GnmCell const *co, GnmCell const *cn)
 {
@@ -813,28 +986,12 @@ compare_corresponding_cells (GnmCell const *co, GnmCell const *cn)
 	if (has_expr != gnm_cell_has_expr (cn))
 		return TRUE;
 	if (has_expr) {
-		char *so, *sn;
-		GnmParsePos ppo, ppn;
-		gboolean eq;
-
-		if (gnm_expr_top_equal (co->base.texpr, cn->base.texpr))
-			return FALSE;
-
-		// Not equal, but with references to sheets, that is not
-		// necessary.  Compare as strings.
-
-		parse_pos_init_cell (&ppo, co);
-		so = gnm_expr_top_as_string (co->base.texpr, &ppo, sheet_get_conventions (co->base.sheet));
-
-		parse_pos_init_cell (&ppn, cn);
-		sn = gnm_expr_top_as_string (cn->base.texpr, &ppn, sheet_get_conventions (cn->base.sheet));
-
-		eq = g_strcmp0 (so, sn) == 0;
-
-		g_free (so);
-		g_free (sn);
-
-		return !eq;
+		GnmParsePos opp, npp;
+		parse_pos_init_cell (&opp, co);
+		parse_pos_init_cell (&npp, cn);
+		return !compare_texpr_equal (co->base.texpr, &opp,
+					     cn->base.texpr, &npp,
+					     sheet_get_conventions (cn->base.sheet));
 	}
 
 	if (has_value != (cn->value != NULL))
@@ -863,13 +1020,13 @@ ignore_cell (GnmCell const *cell)
 }
 
 static void
-diff_sheets_cells (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
+diff_sheets_cells (GnmDiffState *state)
 {
-	GPtrArray *old_cells = sheet_cells (old_sheet, NULL);
-	GPtrArray *new_cells = sheet_cells (new_sheet, NULL);
+	GPtrArray *old_cells = sheet_cells (state->old_sheet, NULL);
+	GPtrArray *new_cells = sheet_cells (state->new_sheet, NULL);
 	size_t io = 0, in = 0;
 
-	/* Make code below simpler.  */
+	// Make code below simpler.
 	g_ptr_array_add (old_cells, NULL);
 	g_ptr_array_add (new_cells, NULL);
 
@@ -916,20 +1073,54 @@ diff_sheets_cells (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
 	g_ptr_array_free (new_cells, TRUE);
 }
 
+static void
+diff_sheets_colrow (GnmDiffState *state, gboolean is_cols)
+{
+	ColRowInfo const *old_def =
+		sheet_colrow_get_default (state->old_sheet, is_cols);
+	ColRowInfo const *new_def =
+		sheet_colrow_get_default (state->new_sheet, is_cols);
+	int i, U;
+
+	if (!colrow_equal (old_def, new_def)) {
+		state->diff_found = TRUE;
+		state->actions->colrow_changed (state, old_def, new_def, is_cols, -1);
+	}
+
+	U = is_cols
+		? state->common_range.end.col
+		: state->common_range.end.row;
+	for (i = 0; i <= U; i++) {
+		ColRowInfo const *ocr =
+			sheet_colrow_get (state->old_sheet, i, is_cols);
+		ColRowInfo const *ncr =
+			sheet_colrow_get (state->new_sheet, i, is_cols);
+
+		if (ocr == ncr)
+			continue; // Considered equal, even if defaults are different
+		if (!ocr) ocr = old_def;
+		if (!ncr) ncr = new_def;
+		if (!colrow_equal (ocr, ncr)) {
+			state->diff_found = TRUE;
+			state->actions->colrow_changed (state, ocr, ncr, is_cols, i);
+		}
+	}
+}
+
 #define DO_INT(field,attr)						\
-	do {								\
-		if (old_sheet->field != new_sheet->field) {		\
-			state->diff_found = TRUE;			\
-			state->actions->sheet_attr_int_changed		\
-				(state, attr, old_sheet->field, new_sheet->field); \
-		}							\
-} while (0)
+  do {									\
+	  if (state->old_sheet->field != state->new_sheet->field) {	\
+		  state->diff_found = TRUE;				\
+		  state->actions->sheet_attr_int_changed		\
+			  (state, attr, state->old_sheet->field, state->new_sheet->field); \
+	  }								\
+  } while (0)
 
 static void
-diff_sheets_attrs (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
+diff_sheets_attrs (GnmDiffState *state)
 {
-	GnmSheetSize const *os = gnm_sheet_get_size (old_sheet);
-	GnmSheetSize const *ns = gnm_sheet_get_size (new_sheet);
+	GnmSheetSize const *os = gnm_sheet_get_size (state->old_sheet);
+	GnmSheetSize const *ns = gnm_sheet_get_size (state->new_sheet);
 
 	if (os->max_cols != ns->max_cols) {
 		state->diff_found = TRUE;
@@ -958,8 +1149,6 @@ diff_sheets_attrs (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
 
 struct cb_diff_sheets_styles {
 	GnmDiffState *state;
-	Sheet const *old_sheet;
-	Sheet const *new_sheet;
 	GnmStyle *old_style;
 };
 
@@ -969,16 +1158,15 @@ cb_diff_sheets_styles_2 (G_GNUC_UNUSED gpointer key,
 {
 	GnmStyleRegion *sr = sr_;
 	struct cb_diff_sheets_styles *data = user_data;
+	GnmDiffState *state = data->state;
 	GnmRange r = sr->range;
 
 	if (gnm_style_find_differences (data->old_style, sr->style, TRUE) == 0)
 		return;
 
-	data->state->diff_found = TRUE;
+	state->diff_found = TRUE;
 
-	data->state->actions->style_changed (data->state, &r,
-					     data->old_sheet, data->new_sheet,
-					     data->old_style, sr->style);
+	state->actions->style_changed (state, &r, data->old_style, sr->style);
 }
 
 static void
@@ -987,41 +1175,107 @@ cb_diff_sheets_styles_1 (G_GNUC_UNUSED gpointer key,
 {
 	GnmStyleRegion *sr = sr_;
 	struct cb_diff_sheets_styles *data = user_data;
+	GnmDiffState *state = data->state;
 
 	data->old_style = sr->style;
-	sheet_style_range_foreach (data->new_sheet, &sr->range,
+	sheet_style_range_foreach (state->new_sheet, &sr->range,
 				   cb_diff_sheets_styles_2,
 				   data);
 }
 
 static void
-diff_sheets_styles (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
+diff_sheets_styles (GnmDiffState *state)
 {
-	GnmSheetSize const *os = gnm_sheet_get_size (old_sheet);
-	GnmSheetSize const *ns = gnm_sheet_get_size (new_sheet);
-	GnmRange r;
 	struct cb_diff_sheets_styles data;
 
-	/* Compare largest common area only.  */
-	range_init (&r, 0, 0,
-		    MIN (os->max_cols, ns->max_cols) - 1,
-		    MIN (os->max_rows, ns->max_rows) - 1);
-
 	data.state = state;
-	data.old_sheet = old_sheet;
-	data.new_sheet = new_sheet;
-	sheet_style_range_foreach (old_sheet, &r,
+	sheet_style_range_foreach (state->old_sheet, &state->common_range,
 				   cb_diff_sheets_styles_1,
 				   &data);
 }
 
+static int
+cb_expr_name_by_name (GnmNamedExpr const *a, GnmNamedExpr const *b)
+{
+	return g_strcmp0 (expr_name_name (a), expr_name_name (b));
+}
+
+static void
+diff_names (GnmDiffState *state,
+	    GnmNamedExprCollection const *onames, GnmNamedExprCollection const *nnames)
+{
+	GSList *old_names = gnm_named_expr_collection_list (onames);
+	GSList *new_names = gnm_named_expr_collection_list (nnames);
+	GSList *lo, *ln;
+	GnmConventions const *convs;
+
+	if (state->new_sheet)
+		convs = sheet_get_conventions (state->new_sheet);
+	else
+		// Hmm...  It's not terribly important where we get them
+		convs = sheet_get_conventions (workbook_sheet_by_index (state->new.wb, 0));
+		
+	old_names = g_slist_sort (old_names, (GCompareFunc)cb_expr_name_by_name);
+	new_names = g_slist_sort (new_names, (GCompareFunc)cb_expr_name_by_name);
+
+	lo = old_names;
+	ln = new_names;
+	while (lo || ln) {
+		GnmNamedExpr const *on = lo ? lo->data : NULL;
+		GnmNamedExpr const *nn = ln ? ln->data : NULL;
+
+		if (!nn || (on && cb_expr_name_by_name (on, nn) < 0)) {
+			// Old name got removed
+			state->diff_found = TRUE;
+			state->actions->name_changed (state, on, nn);
+			lo = lo->next;
+			continue;
+		}
+
+		if (!on || (nn && cb_expr_name_by_name (on, nn) > 0)) {
+			// New name got added
+			state->diff_found = TRUE;
+			state->actions->name_changed (state, on, nn);
+			ln = ln->next;
+			continue;
+		}
+
+		if (!compare_texpr_equal (on->texpr, &on->pos,
+					  nn->texpr, &nn->pos,
+					  convs)) {
+			state->diff_found = TRUE;
+			state->actions->name_changed (state, on, nn);
+		}
+
+		lo = lo->next;
+		ln = ln->next;
+	}
+
+	g_slist_free (old_names);
+	g_slist_free (new_names);
+}
+
+
 static void
 diff_sheets (GnmDiffState *state, Sheet *old_sheet, Sheet *new_sheet)
 {
-	diff_sheets_attrs (state, old_sheet, new_sheet);
-	/* Compare row/column attributes.  */
-	diff_sheets_cells (state, old_sheet, new_sheet);
-	diff_sheets_styles (state, old_sheet, new_sheet);
+	GnmRange or, nr;
+
+	state->old_sheet = old_sheet;
+	state->new_sheet = new_sheet;
+
+	range_init_full_sheet (&or, old_sheet);
+	range_init_full_sheet (&nr, new_sheet);
+	range_intersection (&state->common_range, &or, &nr);
+
+	diff_sheets_attrs (state);
+	diff_names (state, state->old_sheet->names, state->new_sheet->names);
+	diff_sheets_colrow (state, TRUE);
+	diff_sheets_colrow (state, FALSE);
+	diff_sheets_cells (state);
+	diff_sheets_styles (state);
+
+	state->old_sheet = state->new_sheet = NULL;
 }
 
 static int
@@ -1053,11 +1307,10 @@ diff (char const *oldfilename, char const *newfilename,
 	if (state.actions->diff_start (&state))
 		goto error;
 
-	/*
-	 * This doesn't handle sheet renames very well, but simply considers
-	 * that a sheet deletion and a sheet insert.
-	 */
+	diff_names (&state, state.old.wb->names, state.new.wb->names);
 
+	// This doesn't handle sheet renames very well, but simply considers
+	// that a sheet deletion and a sheet insert.
 	count = workbook_sheet_count (state.old.wb);
 	for (i = 0; i < count; i++) {
 		Sheet *old_sheet = workbook_sheet_by_index (state.old.wb, i);
@@ -1071,7 +1324,8 @@ diff (char const *oldfilename, char const *newfilename,
 			last_index = new_sheet->index_in_wb;
 
 			diff_sheets (&state, old_sheet, new_sheet);
-		}
+		} else
+			state.diff_found = TRUE;
 
 		state.actions->sheet_end (&state);
 	}
@@ -1082,9 +1336,10 @@ diff (char const *oldfilename, char const *newfilename,
 		Sheet *old_sheet = workbook_sheet_by_name (state.old.wb,
 							   new_sheet->name_unquoted);
 		if (old_sheet)
-			; /* Nothing -- already done above. */
+			; // Nothing -- already done above.
 		else {
-			state.actions->sheet_start (&state, NULL, new_sheet);
+			state.diff_found = TRUE;
+			state.actions->sheet_start (&state, old_sheet, new_sheet);
 			state.actions->sheet_end (&state);
 		}
 	}
@@ -1099,12 +1354,7 @@ diff (char const *oldfilename, char const *newfilename,
 out:
 	clear_file_state (&state.old);
 	clear_file_state (&state.new);
-	clear_file_state (&state.highlight);
-	g_clear_object (&state.xml);
-	if (state.convs)
-		gnm_conventions_unref (state.convs);
-	if (state.highlight_style)
-		gnm_style_unref (state.highlight_style);
+	state.actions->dtor (&state);
 
 	gnm_pop_C_locale (locale);
 
@@ -1130,7 +1380,7 @@ main (int argc, char const **argv)
 	char *output_uri;
 	GsfOutput *output;
 
-	/* No code before here, we need to init threads */
+	// No code before here, we need to init threads
 	argv = gnm_pre_parse_init (argc, argv);
 
 	ocontext = g_option_context_new (_("OLDFILE NEWFILE"));
@@ -1187,7 +1437,7 @@ main (int argc, char const **argv)
 	go_plugin_db_activate_plugin_list (
 		go_plugins_get_available_plugins (), &plugin_errs);
 	if (plugin_errs) {
-		/* FIXME: What do we want to do here? */
+		// FIXME: What do we want to do here?
 		go_error_info_free (plugin_errs);
 	}
 	go_component_set_default_command_context (cc);
@@ -1203,7 +1453,7 @@ main (int argc, char const **argv)
 		res = 2;
 	}
 
-	/* Release cached string. */
+	// Release cached string.
 	def_cell_name (NULL);
 	g_object_unref (output);
 
