@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <gnumeric-config.h>
@@ -136,44 +135,10 @@ stf_preparse (GOIOContext *context, GsfInput *input, size_t *data_len)
 	return data;
 }
 
-static void
-stf_apply_formats (StfParseOptions_t *parseoptions,
-		   Sheet *sheet, int col, int start_row, int end_row)
-{
-	unsigned int ui;
-	GnmRange range;
-
-	/* If we didn't use the stf dialog, then formats will be NULL */
-	if (parseoptions->formats == NULL)
-		return;
-
-	range.start.col = col;
-	range.start.row = start_row;
-	range.end.col   = col;
-	range.end.row   = end_row;
-
-	for (ui = 0; ui < parseoptions->formats->len; ui++) {
-		if (parseoptions->col_import_array == NULL ||
-		    parseoptions->col_import_array_len <= ui ||
-		    parseoptions->col_import_array[ui]) {
-			GnmStyle *style = gnm_style_new ();
-			GOFormat *sf = g_ptr_array_index
-				(parseoptions->formats, ui);
-			gnm_style_set_format (style, sf);
-			sheet_style_apply_range (sheet, &range, style);
-			range.start.col++;
-			range.end.col++;
-		}
-	}
-}
-
 static gboolean
 stf_store_results (DialogStfResult_t *dialogresult,
 		   Sheet *sheet, int start_col, int start_row)
 {
-	stf_apply_formats (dialogresult->parseoptions,
-			   sheet, start_col, start_row,
-			   start_row + dialogresult->rowcount - 1);
 	return stf_parse_sheet (dialogresult->parseoptions,
 				dialogresult->text, NULL, sheet,
 				start_col, start_row);
