@@ -369,13 +369,10 @@ gnm_style_cond_get_alternate_expr (GnmStyleCond const *cond)
 static gboolean
 isself (GnmExpr const *expr)
 {
-	GnmCellRef const *cr;
+	GnmCellRef const *cr = gnm_expr_get_cellref (expr);
 
-	if (GNM_EXPR_GET_OPER (expr) != GNM_EXPR_OP_CELLREF)
-		return FALSE;
-	cr = &expr->cellref.ref;
-
-	return (cr->sheet == NULL &&
+	return (cr &&
+		cr->sheet == NULL &&
 		cr->col == 0 && cr->row == 0 &&
 		cr->col_relative && cr->row_relative);
 }
@@ -658,7 +655,8 @@ gnm_style_cond_eval (GnmStyleCond const *cond, GnmValue const *cv,
 }
 
 static gboolean
-gnm_style_cond_equal (GnmStyleCond const *ca, GnmStyleCond const *cb)
+gnm_style_cond_equal (GnmStyleCond const *ca, GnmStyleCond const *cb,
+		      gboolean relax_sheet)
 {
 	unsigned oi, N;
 
@@ -670,7 +668,7 @@ gnm_style_cond_equal (GnmStyleCond const *ca, GnmStyleCond const *cb)
 
 	N = gnm_style_cond_op_operands (ca->op);
 	for (oi = 0; oi < N; oi++) {
-		if (ca->deps[oi].sheet != cb->deps[oi].sheet)
+		if (!relax_sheet && ca->deps[oi].sheet != cb->deps[oi].sheet)
 			return FALSE;
 		if (!gnm_expr_top_equal (ca->deps[oi].texpr,
 					 cb->deps[oi].texpr))
@@ -792,12 +790,14 @@ gnm_style_conditions_hash (GnmStyleConditions const *sc)
  * gnm_style_conditions_equal:
  * @sca: first #GnmStyleConditions to compare.
  * @scb: second #GnmStyleConditions to compare.
+ * @relax_sheet: if %TRUE, ignore differences solely caused by being linked into different sheets.
  *
  * Returns: %TRUE if the conditions are equal.
  **/
 gboolean
 gnm_style_conditions_equal (GnmStyleConditions const *sca,
-			    GnmStyleConditions const *scb)
+			    GnmStyleConditions const *scb,
+			    gboolean relax_sheet)
 {
 	GPtrArray const *ga, *gb;
 	unsigned ui;
@@ -805,7 +805,7 @@ gnm_style_conditions_equal (GnmStyleConditions const *sca,
 	g_return_val_if_fail (sca != NULL, FALSE);
 	g_return_val_if_fail (scb != NULL, FALSE);
 
-	if (sca->sheet != scb->sheet)
+	if (!relax_sheet && sca->sheet != scb->sheet)
 		return FALSE;
 
 	ga = gnm_style_conditions_details (sca);
@@ -818,7 +818,7 @@ gnm_style_conditions_equal (GnmStyleConditions const *sca,
 	for (ui = 0; ui < ga->len; ui++) {
 		GnmStyleCond const *ca = g_ptr_array_index (ga, ui);
 		GnmStyleCond const *cb = g_ptr_array_index (gb, ui);
-		if (!gnm_style_cond_equal (ca, cb))
+		if (!gnm_style_cond_equal (ca, cb, relax_sheet))
 			return FALSE;
 	}
 
