@@ -19,8 +19,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <gnumeric-config.h>
@@ -40,7 +39,7 @@ GnmExpr const *analysis_tool_combine_area (GnmValue *val_1, GnmValue *val_2, Wor
 	GnmFunc *fd_array;
 	GnmExpr const *expr;
 
-	if (val_1->type == VALUE_CELLRANGE && val_2->type == VALUE_CELLRANGE &&
+	if (VALUE_IS_CELLRANGE (val_1) && VALUE_IS_CELLRANGE (val_2) &&
 	    val_1->v_range.cell.a.sheet == val_2->v_range.cell.a.sheet) {
 		GnmRange r_1, r_2;
 		gboolean combined = FALSE;
