@@ -34,7 +34,6 @@
 #include "func.h"
 #include "expr.h"
 #include "expr-name.h"
-#include "expr-impl.h"
 #include "value.h"
 #include "ranges.h"
 #include "selection.h"
@@ -341,7 +340,7 @@ wb_view_style_feedback (WorkbookView *wbv)
 		GnmRange const *r;
 		if (NULL == (r = gnm_sheet_merge_contains_pos (sv->sheet, &sv->edit_pos)))
 			r = range_init_cellpos (&corner, &sv->edit_pos);
-		sheet_object_anchor_init (&anchor, r, a_offsets, GOD_ANCHOR_DIR_DOWN_RIGHT);
+		sheet_object_anchor_init (&anchor, r, a_offsets, GOD_ANCHOR_DIR_DOWN_RIGHT, GNM_SO_ANCHOR_TWO_CELLS);
 		sheet_object_set_anchor (wbv->in_cell_combo, &anchor);
 		sheet_object_set_sheet (wbv->in_cell_combo, sv->sheet);
 	}
@@ -469,14 +468,15 @@ wb_view_edit_line_set (WorkbookView *wbv, WorkbookControl *optional_wbc)
 				}
 
 				if (corner) {
-					GnmExprArrayCorner const *ac = gnm_cell_is_array_corner (corner);
+					int cols, rows;
+					char *tmp;
 
-					char *tmp = g_strdup_printf
+					gnm_expr_top_get_array_size (corner->base.texpr, &cols, &rows);
+
+					tmp = g_strdup_printf
 						("{%s}(%d%c%d)[%d][%d]",
 						 text,
-						 ac->cols,
-						 go_locale_get_arg_sep (),
-						 ac->rows,
+						 cols, go_locale_get_arg_sep (), rows,
 						 x, y);
 					g_free (text);
 					text = tmp;
@@ -1308,7 +1308,7 @@ workbook_view_new_from_input (GsfInput *input,
 /**
  * workbook_view_new_from_uri:
  * @uri: URI for file
- * @optional_format: Optional GOFileOpener
+ * @optional_format: (allow-none): Optional GOFileOpener
  * @io_context: Optional context to display errors.
  * @optional_encoding: Optional encoding for GOFileOpener that understand it
  *
