@@ -39,7 +39,6 @@
 #include <style-color.h>
 #include <style-conditions.h>
 #include <expr.h>
-#include <expr-impl.h>
 #include <expr-name.h>
 #include <value.h>
 #include <ranges.h>
@@ -476,7 +475,7 @@ xml_write_style (GnmOutputXML *state, GnmStyle const *style)
 		GNM "Diagonal"
 	};
 	GnmValidation const *v;
-	GnmHLink   const *link;
+	GnmHLink   const *lnk;
 	GnmInputMsg const *im;
 	GnmStyleConditions const *sc;
 	GnmStyleBorderType t;
@@ -558,12 +557,12 @@ xml_write_style (GnmOutputXML *state, GnmStyle const *style)
 	}
 
 	if (gnm_style_is_element_set (style, MSTYLE_HLINK) &&
-	    NULL != (link = gnm_style_get_hlink (style))) {
+	    NULL != (lnk = gnm_style_get_hlink (style))) {
 		gsf_xml_out_start_element (state->output, GNM "HyperLink");
-		gsf_xml_out_add_cstr (state->output, "type", g_type_name (G_OBJECT_TYPE (link)));
-		gsf_xml_out_add_cstr (state->output, "target", gnm_hlink_get_target (link));
-		if (gnm_hlink_get_tip (link) != NULL)
-			gsf_xml_out_add_cstr (state->output, "tip", gnm_hlink_get_tip (link));
+		gsf_xml_out_add_cstr (state->output, "type", g_type_name (G_OBJECT_TYPE (lnk)));
+		gsf_xml_out_add_cstr (state->output, "target", gnm_hlink_get_target (lnk));
+		if (gnm_hlink_get_tip (lnk) != NULL)
+			gsf_xml_out_add_cstr (state->output, "tip", gnm_hlink_get_tip (lnk));
 		gsf_xml_out_end_element (state->output);
 	}
 
@@ -864,8 +863,10 @@ xml_write_cell_and_position (GnmOutputXML *state,
 	/* As of version 0.53 we save the size of the array as attributes */
 	/* As of version 0.57 the attributes are in the Cell not the Content */
 	if (texpr && gnm_expr_top_is_array_corner (texpr)) {
-		gsf_xml_out_add_int (state->output, "Rows", texpr->expr->array_corner.rows);
-		gsf_xml_out_add_int (state->output, "Cols", texpr->expr->array_corner.cols);
+		int cols, rows;
+		gnm_expr_top_get_array_size (texpr, &cols, &rows);
+		gsf_xml_out_add_int (state->output, "Rows", rows);
+		gsf_xml_out_add_int (state->output, "Cols", cols);
 	}
 
 	if (write_contents) {
